@@ -13,8 +13,8 @@
 // TODO: handle gv
 // TODO: Str->utf8_on/Str->utf8_off
 // TODO: Str->len_bytes() / Str->len()
-// TODO: Value->true() # SvTRUE()
 // TODO: Scalar->weaken / Scalar->is_weak
+// TODO: handle signals?
 
 extern "C" {
 #include "EXTERN.h"
@@ -24,6 +24,11 @@ extern "C" {
 #include <cstdarg>
 };
 
+#ifdef _WIN32
+# undef stderr
+# undef stdout
+#endif
+
 namespace pl {
     class Str;
     class UInt;
@@ -125,7 +130,7 @@ namespace pl {
          * @see SvREFCNT_inc
          */
         void refcnt_inc() {
-            SvREFCNT_inc(this->val);
+            SvREFCNT_inc_simple_void(this->val);
         }
         /**
          * decrement the reference counter for this value
@@ -140,6 +145,9 @@ namespace pl {
         int refcnt() {
             return SvREFCNT(this->val);
         }
+        bool is_true() {
+            return SvTRUE(this->val);
+        }
         /**
          * get a reference of this value
          */
@@ -175,12 +183,6 @@ namespace pl {
             sv_2mortal(this->val);
             return this;
         }
-        /**
-         * serialize this object to real perl variable
-         */
-        SV * serialize() {
-            return val;
-        }
         /**
          * this variable is just a string.change the type
          */
@@ -209,37 +211,41 @@ namespace pl {
             return Scalar::create(newSVsv(this->val));
         }
 
-        static Scalar *to_perl(const char* s) {
-            return Scalar::create(newSVpv(s, strlen(s)));
-        }
-        static Scalar *to_perl(unsigned int v) {
-            return Scalar::create(newSVuv(v));
-        }
-        static Scalar *to_perl(int v) {
-            return Scalar::create(newSViv(v));
-        }
-        static Scalar *to_perl(I32 v) {
-            return Scalar::create(newSViv(v));
-        }
-        static Scalar *to_perl(double v) {
-            return Scalar::create(newSVnv(v));
-        }
-        static Scalar *to_perl(Scalar * v) {
-            if (v && v->val) {
-                return Scalar::create(v->val);
-            } else {
-                return Scalar::create(&PL_sv_undef);
-            }
-        }
-        static Scalar *to_perl(std::string& v) {
-            return Scalar::create(newSVpv(v.c_str(), v.length()));
-        }
-        static Scalar *to_perl(bool b) {
-            return Scalar::create(b ? &PL_sv_yes : &PL_sv_no);
+        template <class T>
+        static Scalar *to_perl(T s) {
+            return Scalar::create(to_sv(s));
         }
     protected:
         Scalar(SV* _v) : Value(_v) { }
         static Scalar * create(SV* _v);
+        static SV *to_sv(const char* s) {
+            return (newSVpv(s, strlen(s)));
+        }
+        static SV *to_sv(unsigned int v) {
+            return (newSVuv(v));
+        }
+        static SV *to_sv(int v) {
+            return (newSViv(v));
+        }
+        static SV *to_sv(I32 v) {
+            return (newSViv(v));
+        }
+        static SV *to_sv(double v) {
+            return (newSVnv(v));
+        }
+        static SV *to_sv(Scalar * v) {
+            if (v && v->val) {
+                return (v->val);
+            } else {
+                return (&PL_sv_undef);
+            }
+        }
+        static SV *to_sv(std::string& v) {
+            return (newSVpv(v.c_str(), v.length()));
+        }
+        static SV *to_sv(bool b) {
+            return (b ? &PL_sv_yes : &PL_sv_no);
+        }
     };
 
     /**
@@ -403,10 +409,11 @@ namespace pl {
         /// store value to hash
         template <class T>
         void store(const char*key, T value) {
-            this->store(key, strlen(key), Scalar::to_perl(value));
+            this->store(key, strlen(key), value);
         }
         /// store value to hash
-        void  store(const char*key, I32 klen, Scalar*value);
+        template <class T>
+        void  store(const char*key, I32 klen,  T value);
         /// Evaluates the hash in scalar context and returns the result.
         Scalar* scalar();
         /// Undefines the hash.
@@ -431,9 +438,9 @@ namespace pl {
         }
         template <class T>
         void push(T v) {
-            Scalar * s = Scalar::to_perl(v);
-            s->refcnt_inc();
-            av_push((AV*)this->val, s->val);
+            SV * s = Scalar::to_sv(v);
+            SvREFCNT_inc_simple_void(s);
+            av_push((AV*)this->val, s);
         }
         /**
           * Unshift the given number of "undef" values onto the beginning
@@ -462,7 +469,8 @@ namespace pl {
         }
 
         /// store values to array
-        Scalar * store(I32 key, Scalar* v);
+        template <class T>
+        Scalar * store(I32 key, T v);
         /// Clears an array, making it empty.
         void clear() {
             av_clear((AV*)this->val);
@@ -500,12 +508,12 @@ namespace pl {
         /// return the one scalar value
         template <class T>
         void ret(T n) {
-            Scalar * s = Scalar::to_perl(n);
-            this->ret(0, s->val);
+            SV * s = Scalar::to_sv(n);
+            this->ret(0, s);
         }
         template <class T>
         void ret(int n, T v) {
-            return this->ret(n, Scalar::to_perl(v));
+            return this->ret(n, Scalar::to_sv(v));
         }
         /// same as perl level wantarray()
         bool wantarray() {
@@ -627,9 +635,11 @@ namespace pl {
         SV* v = av_shift((AV*)this->val);
         return Scalar::create(v);
     }
-    Scalar * Array::store(I32 key, Scalar* _v) {
-        _v->refcnt_inc();
-        SV** v = av_store((AV*)this->val, key, _v->val);
+    template <class T>
+    Scalar * Array::store(I32 key, T arg) {
+        SV * _v = Scalar::to_sv(arg);
+        SvREFCNT_inc_simple_void(_v);
+        SV** v = av_store((AV*)this->val, key, _v);
         if (v) {
             Reference * ref = new Reference(*v);
             CurCtx::get()->register_allocated(ref);
@@ -644,17 +654,18 @@ namespace pl {
      */
     class Package {
     public:
-        Package(std::string _pkg) {
-            pkg = _pkg;
+        Package(std::string _pkg, const char *_file) {
+            this->pkg = _pkg;
+            this->file = _file;
             stash = gv_stashpvn(pkg.c_str(), pkg.length(), TRUE);
         }
         /**
          * install xsub
          * @see newXS
          */
-        void add_method(const char*name, XSUBADDR_t func, const char *file) {
+        void add_method(const char*name, XSUBADDR_t func) {
             char * buf = const_cast<char*>( (pkg + "::" + name).c_str() );
-            newXS(buf, func, const_cast<char*>(file));
+            newXS(buf, func, const_cast<char*>(this->file));
         }
         /**
          * add new const sub
@@ -666,8 +677,8 @@ namespace pl {
 
         template <class T>
         void add_constant(const char *name, T val) {
-            Scalar * s = Scalar::to_perl(val);
-            this->add_constant(name, s->serialize());
+            SV * s = Scalar::to_sv(val);
+            this->add_constant(name, s);
         }
     protected:
         void add_constant(const char *name, SV* val) {
@@ -676,6 +687,7 @@ namespace pl {
     private:
         std::string pkg;
         HV * stash;
+        const char * file;
     };
 
     /**
@@ -991,9 +1003,11 @@ namespace pl {
         CurCtx::get()->register_allocated(ref);
         return ref;
     }
-    void Hash::store(const char*key, I32 klen, Scalar*value) {
-        value->refcnt_inc();
-        hv_store((HV*)this->val, key, klen, value->val, 0);
+    template <class T>
+    void Hash::store(const char*key, I32 klen, T value) {
+        SV * v = Scalar::to_sv(value);
+        SvREFCNT_inc_simple_void(v);
+        hv_store((HV*)this->val, key, klen, v, 0);
     }
     Scalar* Hash::scalar() {
         Scalar*s = new Scalar(hv_scalar((HV*)this->val));
